// Copyright 2015 Krajowa Izba Rozliczeniowa. All rights reserved.

var _szafirConns = [],
		_szafirHostID = "pl.com.kir.szafirhost",
		_versionInfo = {
			required: "VERSION:1.0.5",
			app_links: {
				windows: "http://www.elektronicznypodpis.pl/gfx/elektronicznypodpis/pl/defaultstronaopisowa/146/1/1/szafirhost.msi",
				linux: "http://www.elektronicznypodpis.pl/gfx/elektronicznypodpis/pl/defaultstronaopisowa/146/1/1/szafirhost-install.jar",
				macos: "http://www.elektronicznypodpis.pl/gfx/elektronicznypodpis/pl/defaultstronaopisowa/146/1/1/szafirhost-install.jar"
			}
		};

function logJson(jsonMsg) {
	var msg = JSON.stringify(jsonMsg);
	return msg.substring(0, 100) + (msg.length > 100 ? "..." : "");
}

function parseVersion(versionStr) {
	var m = versionStr.match(/VERSION:(\d+)\.(\d+)\.(\d+)/);
	if (m !== null) {
		return { major: parseInt(m[1]), minor: parseInt(m[2]), patch: parseInt(m[3]) };
	}
	return null;
}

function isHostUpdateRequired(versionStr) {
	var verRequired = parseVersion(_versionInfo.required);
	var currVer = parseVersion(versionStr) || parseVersion("VERSION:0.0.0");

	if (((verRequired.major * 1000000) + (verRequired.minor * 1000) + verRequired.patch) >
			((currVer.major * 1000000) + (currVer.minor * 1000) + currVer.patch)) {
		return true;
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// BEGIN SzafirConnection

function SzafirConnection(tabId, tabUrl, jsonConfig) {
	this._loaded = false;
	this._tabId = tabId;
	this._tabUrl = tabUrl;
	this._config = jsonConfig;
	this._nativePort = null;
	this._nativeVersion = null;
	console.log("szconn.constructor -> TabID: " + this._tabId);
}

SzafirConnection.prototype.getTabId = function() {
	return this._tabId;
}

SzafirConnection.prototype.getTabUrl = function() {
	return this._tabUrl;
}

SzafirConnection.prototype.isNewUrl = function(updatedUrl) {
	var noHashTagUrl = (updatedUrl || "").split("#")[0];
	return (this._tabUrl.indexOf(noHashTagUrl) != 0);
}

SzafirConnection.prototype.loadSzafir = function(responseCallback) {
	console.log("szconn.loadSzafir -> TabID: " + this._tabId);
	var that = this;

	if (this._nativePort == null) {
		this._nativePort = chrome.runtime.connectNative(_szafirHostID);
		console.log("PORT: " + this._nativePort);

		this._nativePort.onMessage.addListener(function(jsonMsg) {
			that.onMessageFromNative(jsonMsg);
		});
		this._nativePort.onDisconnect.addListener(function(event) {
			that.onNativeDisconnected(event);
		});

	} else {
		//responseCallback({status: true, error: "no error"});
	}
}

SzafirConnection.prototype.isConnected = function() {
	console.log("szconn.isConnected -> TabID: " + this._tabId);
	return (this._nativePort !== null);
}

SzafirConnection.prototype.unloadSzafir = function() {
	console.log("szconn.unloadSzafir -> TabID: " + this._tabId);
	if (this._nativePort != null) {
		this._nativePort.disconnect();
		this._nativePort = null;
		chrome.pageAction.hide(this._tabId);
	}
	this._loaded = false;
}

SzafirConnection.prototype.sendMessageToNative = function(jsonMsg) {
	console.log("szconn.sendMessageToNative -> TabID: " + this._tabId + "; MSG: " + logJson(jsonMsg));
	if (this._nativePort != null) {
		this._nativePort.postMessage(jsonMsg);
	}
}

SzafirConnection.prototype.onMessageFromNative = function(jsonMsg) {
	console.log("szconn.onMessageFromNative -> TabID: " + this._tabId + "; MSG: " + logJson(jsonMsg));
	chrome.tabs.update(this._tabId, {active: true});

	if (jsonMsg.command === "open") {

		this._nativeVersion = jsonMsg.result;
		console.log(this._nativeVersion);

		if (isHostUpdateRequired(this._nativeVersion)) {
			this.unloadSzafir();
			this.onNativeDisconnected();
		} else {
			var jsonParams = $.extend({command: "load"}, this._config);
			this.sendMessageToNative(jsonParams);
		}

	} else {
		if (jsonMsg.command === "load") {
			chrome.pageAction.show(this._tabId);
			this._loaded = true;
		}
		this.sendMessageToContent(jsonMsg);
	}
}

SzafirConnection.prototype.onNativeDisconnected = function(chrome_event) {
	console.log("szconn.onNativeDisconnected -> TabID: " + this._tabId);
	this._nativePort = null;
	chrome.pageAction.hide(this._tabId);

	if (this._loaded) {
		this.sendMessageToContent({command: "unload", source: "native"});
	} else {
		var versionInfo = $.extend({current: this._nativeVersion}, _versionInfo);
		this.sendMessageToContent({command: "load", status: "ERROR", result: "NOHOSTAPP", version_info: versionInfo });
	}
	this._loaded = false;
}

SzafirConnection.prototype.sendMessageToContent = function(jsonMsg) {
	console.log("szconn.sendMessageToContent -> TabID: " + this._tabId + "; MSG: " + logJson(jsonMsg));
	chrome.tabs.sendMessage(this._tabId, jsonMsg);
}

// END SzafirConnection
////////////////////////////////////////////////////////////////////////////////////////////////////

function getSzafirTab(tabId) {
	for (i = 0; i < _szafirConns.length; i++) {
		if (_szafirConns[i].getTabId() === tabId) {
			return _szafirConns[i];
		}
	}
	return undefined;
}

function createSzafirConnection(tab, jsonConfig) {
	var tabId = tab.id;
	var szafirConn = getSzafirTab(tabId);
	if (szafirConn !== undefined) {
		removeSzafirConnection(tabId);
	}
	szafirConn = new SzafirConnection(tabId, tab.url, jsonConfig);
	_szafirConns.push(szafirConn);
	return szafirConn;
}

function removeSzafirConnection(tabId) {
	console.log("removeSzafirConnection(tabId: " + tabId + ")");
	for (i = 0; i < _szafirConns.length; i++) {
		if (_szafirConns[i].getTabId() === tabId) {
			_szafirConns[i].unloadSzafir();
			_szafirConns.splice(i, 1);
			return true;
		}
	}
	return false;
}

function onMessage(jsonMsg, sender, callbackFunction) {
	var console_log = logJson(jsonMsg);
	if (sender.tab === undefined) {
		console.log("onMessage: START: PageAction Call: " + console_log);

	} else {
		console.log("onMessage: START: " + console_log + ", sender.tab.id = " + sender.tab.id);

		var szafirConn = getSzafirTab(sender.tab.id);
		if (jsonMsg.command === "load") {
			szafirConn = createSzafirConnection(sender.tab, jsonMsg);
			szafirConn.loadSzafir();

		} else if (szafirConn === undefined) {
			error_msg = {command: "error", info: "not connected"};
			console.log("onMessage: " + JSON.stringify(error_msg));
			callbackFunction(error_msg);

		} else if (jsonMsg.command === "unload") {
			removeSzafirConnection(sender.tab.id);

		} else { // funkcje Szafir SDK
			szafirConn.sendMessageToNative(jsonMsg);
		}
	}

	console.log("onMessage: END");
}

function onInstalled() {
	console.log("***** Szafir SDK Browser Extension installed *****");
	chrome.tabs.query({url: "*://*/*"}, function(tabs) {
		for (i = 0; i < tabs.length; i++) {
			chrome.tabs.reload(tabs[i].id);
		}
	});
}

function onCreatedTab(tab) {
	console.log("onCreatedTab(tabId: " + tab.id + ")");
}

function onRemovedTab(tabId, removeInfo) {
	console.log("onRemovedTab(tabId: " + tabId + ")");
	removeSzafirConnection(tabId);
}

function onUpdatedTab(tabId, changeInfo, tab) {
	if (changeInfo.status === "loading" && changeInfo.url !== undefined) {
		console.log("onUpdatedTab(tabId: " + tabId + ", status: " + changeInfo.status + ", url: " + changeInfo.url + ")");
		var szafirTab = getSzafirTab(tabId);
		if (szafirTab !== undefined && szafirTab.isNewUrl(changeInfo.url)) {
			removeSzafirConnection(tabId);
		}
	}
}

function onReplacedTab(addedTabId, removedTabId) {
	console.log("onReplacedTab(addedTabId: " + addedTabId + ", removedTabId: " + removedTabId + ")");
	removeSzafirConnection(removedTabId);
}

if (chrome.runtime.onInstalled) {
	chrome.runtime.onInstalled.addListener(onInstalled);
}
chrome.runtime.onMessage.addListener(onMessage);
chrome.tabs.onCreated.addListener(onCreatedTab);
chrome.tabs.onRemoved.addListener(onRemovedTab);
chrome.tabs.onUpdated.addListener(onUpdatedTab);
chrome.tabs.onReplaced.addListener(onReplacedTab);
