// Copyright 2015 Krajowa Izba Rozliczeniowa. All rights reserved.

var _nativeConnected = false,
		_debug = false,
		_errorHostNotStarted = "Szafir SDK Chrome Host nie został uruchomiony",
		_resultBuffer = "";

var _commands = { load: "load", unload: "unload",
		startComponentURI: "startComponentURI", startComponent: "startComponent",
		getTaskListURI: "getTaskListURI", getTaskList: "getTaskList",
		doTaskList: "doTaskList", doTaskListNoWindow: "doTaskListNoWindow", doTaskListNoWindow1Pass: "doTaskListNoWindow1Pass", doTaskListNoWindow2Pass: "doTaskListNoWindow2Pass",
		doTaskListURI: "doTaskListURI", doTaskListNoWindowURI: "doTaskListNoWindowURI", doTaskListNoWindow1PassURI: "doTaskListNoWindow1PassURI", doTaskListNoWindow2PassURI: "doTaskListNoWindow2PassURI",
		isCardInReader: "isCardInReader", getSignatureInfo: "getSignatureInfo", listCertificates: "listCertificates",
		test: "test",
		partAck: "partAck"},
		_status = { ok: "OK", error: "ERROR" };

function debugLog(log) {
	if (_debug === true) {
		console.log(log);
	}
}

function WND_postMsg(params) {
	var msg = $.extend(params, { type: "SZAFIR_WND_MSG" }); //, status: _status.ok
	window.postMessage(JSON.stringify(msg), "*");
}

function WND_postErrorMsg(params) {
	var msg = $.extend(params, { type: "SZAFIR_WND_MSG", status: _status.error });
	window.postMessage(JSON.stringify(msg), "*");
}

function handleWindowMessages() {
	window.addEventListener("message", function(e) {
		debugLog("CNT.js : WND MESSAGE");

		// We only accept messages from ourselves
		if (e.source != window) {
			return;
		}

		var data = JSON.parse(e.data);
		if (data.type && (data.type == "SZAFIR_EXT_MSG")) {

			switch (data.params.command) {
				case _commands.load:
					NEW_loadSzafir(data.params);
					break;
				case _commands.unload:
					NEW_unloadSzafir();
					break;
				default:
					NEW_commonSzafirFn(data.params);
					break;
			}
		}
	});
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// BEGIN - background.js communication

function handleNativeMessages(jsonResp, sender, callbackFunction) {
	debugLog("CNT.js : NATIVE MESSAGE: " + JSON.stringify(jsonResp));

	switch (jsonResp.command) {
		case _commands.load:
			onLoad(jsonResp);
			break;
		case _commands.unload:
			onUnload(jsonResp);
			break;
		case "switchConsole":
			chrome.runtime.sendMessage({command: "switchConsole", text: "Przełącz konsolę Szafir SDK!!!"});
			callbackFunction({enabled: "enabled"});
			break;
		default:
			NEW_onCommonSzafirResp(jsonResp);
			break;
	}
}

function onLoad(jsonResp) {
	_nativeConnected = (jsonResp.status === _status.ok);
	if (_nativeConnected) {
		jsonResp.result = "LOADED";
	} else {
		jsonResp.status = _status.ok;
		jsonResp.result = (jsonResp.result === "NOHOSTAPP" ? "NOHOSTAPP" : "ERROR");
		// komunikat zawiera też current_version i required_version
	}
	WND_postMsg(jsonResp);
}

function onUnload(jsonResp) {
	_nativeConnected = false;
	jsonResp.result = "UNLOADED";
	WND_postMsg(jsonResp);
}

function NEW_onCommonSzafirResp(jsonResp) {
	if (jsonResp.command == _commands.isCardInReader) {
		jsonResp.result = (jsonResp.result === "true");
	}
	if (jsonResp.part === undefined) {
		WND_postMsg(jsonResp);

	} else {
		debugLog("CNT.js : MULTIPART RESULT, PART: " + jsonResp.part);
		_resultBuffer += jsonResp.result;
		if (jsonResp.part <= 0) {
			jsonResp.result = _resultBuffer;
			_resultBuffer = "";
			WND_postMsg(jsonResp);
		} else {
			NEW_commonSzafirFn({command: _commands.partAck});
		}
	}
}

// END = background.js communication
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// BEGIN - SDK functions

function NEW_loadSzafir(params) {
	_debug = params.config.debug || false;
	debugLog("CNT.js : NEW_loadSzafir");
	var params_json = $.extend({command: _commands.load}, params.config);
	chrome.runtime.sendMessage(params_json);
}

function NEW_unloadSzafir() {
	debugLog("CNT.js : NEW_unloadSzafir");
	if (_nativeConnected) {
		chrome.runtime.sendMessage({command: _commands.unload});
		handleNativeMessages({command: _commands.unload});
	} else {
		WND_postErrorMsg({command: _commands.unload, result: _errorHostNotStarted});
	}
}

function NEW_commonSzafirFn(params) {
	debugLog("CNT.js : NEW_commonSzafirFn -> command: " + (params.command || "NO_COMMAND"));
	if (_nativeConnected) {
		chrome.runtime.sendMessage(params);
	} else {
		WND_postErrorMsg({command: params.command, result: _errorHostNotStarted});
	}
}

///// OLD OLD OLD OLD OLD /////

function OLD_loadSzafir() {
	debugLog("CNT.js : OLD_loadSzafir");
	var params = JSON.parse($("#szafirsdk-params").val());
	var params_json = $.extend({command: _commands.load}, params);
	chrome.runtime.sendMessage(params_json);
}

function OLD_unloadSzafir() {
	debugLog("CNT.js : OLD_unloadSzafir");
	chrome.runtime.sendMessage({command: _commands.unload});
	handleNativeMessages({command: _commands.unload});
}

function OLD_startComponentURI() {
	debugLog("CNT.js : OLD_startComponentURI");
	if (_nativeConnected) {
		var settings_uri = $("#szafirsdk-settings-uri").val();

		$.ajax({url: settings_uri, dataType: "text",
			success: function(settings_cryptoproviders) {
				chrome.runtime.sendMessage({command: _commands.startComponent, settings_xml: settings_cryptoproviders});
			},
			error: function(jqXHR, textStatus, errorThrown) {
				WND_postErrorMsg({command: _commands.startComponent, result: errorThrown});
			}
		});

	} else {
		WND_postErrorMsg({command: _commands.startComponent, result: _errorHostNotStarted});
	}
}

function OLD_startComponent() {
	debugLog("CNT.js : OLD_startComponent");
	if (_nativeConnected) {
		var settings_cryptoproviders = $("#szafirsdk-settings-xml").val();
		chrome.runtime.sendMessage({command: _commands.startComponent, settings_xml: settings_cryptoproviders});
	} else {
		WND_postErrorMsg({command: _commands.startComponent, result: _errorHostNotStarted});
	}
}

function OLD_getTaskListURI() {
	debugLog("CNT.js : OLD_getTaskListURI");
	if (_nativeConnected) {

		var maketasklist_uri = $("#szafirsdk-maketasklist-uri").val();
		$.ajax({url: maketasklist_uri, dataType: "text",
			success: function(maketasklist) {
				chrome.runtime.sendMessage({command: _commands.getTaskList, maketasklist_xml: maketasklist});
			},
			error: function(jqXHR, textStatus, errorThrown) {
				WND_postErrorMsg({command: _commands.getTaskList, result: errorThrown});
			}
		});

	} else {
		WND_postErrorMsg({command: _commands.getTaskList, result: _errorHostNotStarted});
	}
}

function OLD_getTaskList() {
	debugLog("CNT.js : OLD_getTaskList");
	if (_nativeConnected) {
		var maketasklist = $("#szafirsdk-makeTaskList-xml").val();
		chrome.runtime.sendMessage({command: _commands.getTaskList, maketasklist_xml: maketasklist});
	} else {
		WND_postErrorMsg({command: _commands.getTaskList, result: _errorHostNotStarted});
	}
}

function OLD_doTaskList() {
	debugLog("CNT.js : OLD_doTaskList");
	if (_nativeConnected) {
		var tasklist = $("#szafirsdk-taskList-xml").val();
		chrome.runtime.sendMessage({command: _commands.doTaskList, tasklist_xml: tasklist});
	} else {
		WND_postErrorMsg({command: _commands.doTaskList, result: _errorHostNotStarted});
	}
}

function OLD_doTaskListNoWindow() {
	debugLog("CNT.js : OLD_doTaskListNoWindow");
	if (_nativeConnected) {
		var tasklist = $("#szafirsdk-taskList-xml").val();
		chrome.runtime.sendMessage({command: _commands.doTaskListNoWindow, tasklist_xml: tasklist});
	} else {
		WND_postErrorMsg({command: _commands.doTaskListNoWindow, result: _errorHostNotStarted});
	}
}

function OLD_doTaskListNoWindow1Pass() {
	debugLog("CNT.js : OLD_doTaskListNoWindow1Pass");
	if (_nativeConnected) {
		var tasklist = $("#szafirsdk-taskList-xml").val();
		var sigPass = $("#szafirsdk-taskList-pass1").val();
		chrome.runtime.sendMessage({command: _commands.doTaskListNoWindow, tasklist_xml: tasklist, sig_pass: sigPass});
	} else {
		WND_postErrorMsg({command: _commands.doTaskListNoWindow, result: _errorHostNotStarted});
	}
}

function OLD_doTaskListNoWindow2Pass() {
	debugLog("CNT.js : OLD_doTaskListNoWindow2Pass");
	if (_nativeConnected) {
		var tasklist = $("#szafirsdk-taskList-xml").val();
		var sigPass = $("#szafirsdk-taskList-pass1").val();
		var tssPass = $("#szafirsdk-taskList-pass2").val();
		chrome.runtime.sendMessage({command: _commands.doTaskListNoWindow, tasklist_xml: tasklist, sig_pass: sigPass, tss_pass: tssPass});
	} else {
		WND_postErrorMsg({command: _commands.doTaskListNoWindow, result: _errorHostNotStarted});
	}
}

function OLD_isCardInReader() {
	debugLog("CNT.js : OLD_isCardInReader");
	if (_nativeConnected) {
		chrome.runtime.sendMessage({command: _commands.isCardInReader});
	} else {
		WND_postErrorMsg({command: _commands.isCardInReader, result: _errorHostNotStarted});
	}
}

function OLD_getSignatureInfo() {
	debugLog("CNT.js : OLD_getSignatureInfo");
	if (_nativeConnected) {
		var uri = $("#szafirsdk-signature-uri").val();
		var exportData = false;
		chrome.runtime.sendMessage({command: _commands.getSignatureInfo, signature_uri: uri, export_data: exportData});
	} else {
		WND_postErrorMsg({command: _commands.getSignatureInfo, result: _errorHostNotStarted});
	}
}

function OLD_listCertificates() {
	debugLog("CNT.js : OLD_listCertificates");
	if (_nativeConnected) {
		var type_value = $("#szafirsdk-certificate-type").val();
		chrome.runtime.sendMessage({command: _commands.listCertificates, type: type_value});
	} else {
		WND_postErrorMsg({command: _commands.listCertificates, result: _errorHostNotStarted});
	}
}

function OLD_test() {
	debugLog("CNT.js : OLD_test");
	if (_nativeConnected) {
		var test_data = $("#szafirsdk-test-input").val();
		chrome.runtime.sendMessage({command: _commands.test, data: test_data});
	} else {
		WND_postErrorMsg({command: _commands.test, result: _errorHostNotStarted});
	}
}

// END - SDK functions
////////////////////////////////////////////////////////////////////////////////////////////////////

function OLD_init() {
	chrome.runtime.onMessage.addListener(handleNativeMessages);
	handleWindowMessages();

	if ($("#szafirsdk-mode").val() == "SZAFIR_SDK_EXT") { //SZAFIR_SDK_APPLET

		var _szafirConfig = JSON.parse($("#szafirsdk-params").val()) || {};

		$("#szafirsdk-loadSzafir-action").click(OLD_loadSzafir);
		$("#szafirsdk-unloadSzafir-action").click(OLD_unloadSzafir);

		$("#szafirsdk-startComponentUri-action").click(OLD_startComponentURI);
		$("#szafirsdk-startComponent-action").click(OLD_startComponent);

		$("#szafirsdk-getTaskListUri-action").click(OLD_getTaskListURI);
		$("#szafirsdk-getTaskList-action").click(OLD_getTaskList);

		$("#szafirsdk-doTaskList-action").click(OLD_doTaskList);
		$("#szafirsdk-doTaskListNoWindow-action").click(OLD_doTaskListNoWindow);
		$("#szafirsdk-doTaskListNoWindow_1pass-action").click(OLD_doTaskListNoWindow1Pass);
		$("#szafirsdk-doTaskListNoWindow_2pass-action").click(OLD_doTaskListNoWindow2Pass);

		$("#szafirsdk-isCardInReader-action").click(OLD_isCardInReader);
		$("#szafirsdk-getSignatureInfo-action").click(OLD_getSignatureInfo);
		$("#szafirsdk-listCertificates-action").click(OLD_listCertificates);

		$("#szafirsdk-test-action").click(OLD_test);

		if (_szafirConfig.load_szafir_immediately) {
			OLD_loadSzafir();
		}
	}
}

function NEW_init() {
	chrome.runtime.onMessage.addListener(handleNativeMessages);
	handleWindowMessages();
	document.body.dataset.szafirsdkReady = "EXT_INITIALIZED";
}

$(document).ready(function() {

	if ($("#szafirsdk-applet-placeholder").length > 0) {
		console.log("CNT.js : OLD_init");
		OLD_init();

	} else if ($("body").data("szafirsdk-ready")) {
		console.log("CNT.js : NEW_init");
		NEW_init();

	} else {
		console.log("CNT.js : UWAGA -> strona niezgodna z Szafir Sdk");
	}

});
